#include asm86.inc
#include ram86.inc
#include key86.inc
#define NOGROUP
#define NOGETSTORE

.org _asm_exec_ram

	nop
	jp start
	.dw 2				;version 2
	.dw 0				;no comment
	.dw 0				;no icon
	.dw datatypes

start:
	;* generate program table *
	call yas_refresh

	;* standard program start stuff *
	call _clrLCD
	call _flushallmenus
	call _runindicoff

	;* put title at the top of the screen *
	call _homeup
	ld hl,titletext
	call _puts

	;* set initail text, line coordinates *
	ld hl,1+(256*8)
	ld (_penCol),hl
	ld hl,$fc84
	ld (lineaddr),hl

	;* check if there are programs to display *
	call checknumprogs
	or a
	jp z,noprograms

	;* make sure cursor isn't off the end of the list *
	ld hl,cursor
	ld b,(hl)
	dec a
	cp b
	jr nc,cursorok
	ld (hl),a
cursorok:
	inc a

	;* display programs, comments, and draw line *
	ld hl,(progbase)
	ld h,16
	call _htimesl
	ld de,prog_space
	add hl,de
display_progs:
		push af
		push hl
		push hl
		ld de,7
		add hl,de
		call _vputs		
		call drawline
		ld hl,_penCol
		ld (hl),37
		ex (sp),hl
		call yas_comment
		call nc,_vputs
		call _ram_page_1
		pop hl
		ld (hl),1
		inc hl
		ld a,(hl)
		add a,6
		ld (hl),a
		pop hl
		ld de,16
		add hl,de
		pop af
		dec a
		jr nz,display_progs
	ld b,1
	call drawline_v

	;* draw cursor *
	call drawcursor

	;* check keypresses *
keyloop:
	halt
	call _getcsc
	cp K_EXIT
	jp z,_clrWindow
	cp K_MORE
	jr z,nextpage
	cp K_SECOND
	jr z,exec
	cp K_ENTER
	jr z,exec
	call keycheck
	jr keyloop

keycheck:
	;* routine to respond to various keypresses *
	ld hl,_contrast
	cp K_PLUS
	jr z,contrastup
	cp K_MINUS
	jr z,contrastdown
	ld hl,cursor
	cp K_UP
	jr z,cursorup
	dec a			;cp K_DOWN
	ret nz

cursordown:
	;* moves cursor down a line *
	push hl
	call checknumprogs
	pop hl
	dec a
	cp (hl)
	ret z
	call drawcursor
	inc (hl)
jpdrawc:
	jp drawcursor

cursorup:
	;* moves the cursor up a line*
	ld a,(hl)
	or a
	ret z
	call drawcursor
	dec (hl)
	jr jpdrawc

exec:
	;* runs program at cursor *
	call yas_writeback
	ld a,(cursor)
progbase equ $+1
	ld l,0
	add a,l
	ld l,a
	ld h,16
	call _htimesl
	ld de,prog_space
	add hl,de
	call yas_execute
execwait:
	call _getcsc
	or a
	jr nz,execwait
	jp start

contrastup:
	ld a,(hl)
	cp $1f
	ret nc
	inc (hl)
contrastmain:
	ld a,(hl)
	out (2),a
	ret

contrastdown:
	ld a,(hl)
	or a
	ret z
	dec (hl)
	jr contrastmain

nextpage:
	ld hl,(num_progs)
	ld de,progbase
	ld a,(de)
	add a,9
	cp l
	jr c,gotonext
	xor a
gotonext:
	ld (de),a
	xor a
	ld (cursor),a
	jp start

drawcursor:
	push hl
	;* routine to draw the cursor *
cursor equ $+1
	ld l,0
	ld h,16*6
	call _htimesl
	ld de,$fc80
	add hl,de
	ld c,7
	ld de,12
outer_curloop:
		ld b,4
inner_curloop:
			ld a,(hl)
			cpl
			ld (hl),a
			inc l
			djnz inner_curloop
		add hl,de
		dec c
		jr nz,outer_curloop
	pop hl
	ret

drawline:
	;* routine to draw a line down the middle of the screen *
	ld b,6
drawline_v:
lineaddr equ $+1
	ld hl,0
	ld de,16
drawlineloop:
		ld a,(hl)
		or %00100000
		ld (hl),a
		add hl,de
		djnz drawlineloop
	ld (lineaddr),hl
	ret

checknumprogs:
	;* returns number of programs to display *
	ld a,(num_progs)
	ld hl,(progbase)
	sub l
	ret z
	cp 9
	ret c	
	ld a,9
	ret

noprograms:
	;* displays "* no programs *" *
	;or moves to the first page
	ld hl,progbase
	xor a
	ld b,(hl)
	ld (hl),a
	cp b
	jp nz,start
	ld hl,noprogstext
	call _vputs
noprogsloop:	
		halt
		call _getcsc
		or a
		jp nz,_clrWindow
		jr noprogsloop

noprogstext:
	.db "* no programs *",0

titletext:
	.db "YShell    Version 4.0",0

#include yas.asm

datatypes:
	.db 2
	.dw asmdt
	.dw basdt

asmdt:
#include asm_dt_info.asm

basdt:
#include bas_dt_info.asm



.end
